import deskbar, deskbar.Handler
import gnomevfs

from gettext import gettext as _

######################
# User configuration #
######################
DELICIOUS_USER  = None
DELICIOUS_PASS  = None

MAX_QUERIES = 10
QUERY_DELAY = 1


def _check_requirements():
    #We need user and password
    if DELICIOUS_USER is None or DELICIOUS_PASS is None:
        return (deskbar.Handler.HANDLER_HAS_REQUIREMENTS, _("You need to configure this handler."), _on_more_information)
    else:
        return (deskbar.Handler.HANDLER_IS_HAPPY, None, None)


HANDLERS = {
    "DeliciousHandler" : {
        "name": _("delicious"),
        "description": _("Search your delicious' bookmarks by tag name"),
        "requirements" : _check_requirements
    }
}

CONFIG_TEXT = _("""You have to configure your delicious user and password in the handler file.""")

def _on_more_information():
    deskbar.Utils.more_information_dialog(_("Setting Up delicious Handler"), CONFIG_TEXT)


class DeliciousMatch(deskbar.Match.Match):
    
    def __init__(self, handler, name, deliciousMatch, **args):
        deskbar.Match.Match.__init__ (self, handler, name=name, **args)
        self._deliciousInfo = deliciousMatch
        
    def get_verb(self):
        tags=','.join(self._deliciousInfo.tags)
        infoString="%s (tags: %s)" %(self._deliciousInfo.description,tags)
        return infoString

    def action(self, text=None):
        gnomevfs.url_show(self._deliciousInfo.href)

    def get_category(self):
        return "web"

    def get_hash(self, text=None):
        return self._deliciousInfo


    
class DeliciousHandler(deskbar.Handler.AsyncHandler):
    
    def __init__(self):
        deskbar.Handler.AsyncHandler.__init__ (self, "delicious.png")
        self._delicious=DeliciousTagQueryEngine()

    def query(self, tagQuery):
        #Hey man, calm down and query once a time :P
        self.check_query_changed (timeout=QUERY_DELAY)
        
        qmax = min (deskbar.DEFAULT_RESULTS_PER_HANDLER, MAX_QUERIES)
        
        # Yes, the google and yahoo search might take a long time
        # and of course deliciuos too !!! ... better check if we're still valid    
        self.check_query_changed ()
        
        #The queryyyyYyyYy :)
        print "Asking del.icio.us tags for %s" %tagQuery
        posts=self._delicious.get_posts_by_tag(tagQuery)
        
        matchs=[]
        for post in posts:
            if (len(matchs)+1) is qmax:
                break
            matchs.append(DeliciousMatch(self,post.href,post))
        
        self.check_query_changed ()
        
        print "Returning delicious matchs"
        return matchs



import urllib2,base64,libxml2,sys

class DeliciousTagQueryEngine:
    
    def __init__(self):
        """We need use the globals DELICIOUS_USER and DELICIOUS_PASS"""
        self._user=DELICIOUS_USER
        self._pass=DELICIOUS_PASS
        self._DEFAULT_QUERY_TAG="http://del.icio.us/api/posts/all?tag"
        
    def get_posts_by_tag(self,tag):
        url="%s=%s" %(self._DEFAULT_QUERY_TAG,tag)
        #Get the info from del.icio.us and parse
        xml=libxml2.parseDoc(self._get_delicious_url(url))
        postsXML = xml.xpathEval("/posts/post")
        
        #And return the results
        posts=[]
        for post in postsXML:
            posts.append(DeliciousPost(post))  
        
        return posts
        
    def _get_delicious_url(self,url):
        req = urllib2.Request(url)
        try:
            handle = urllib2.urlopen(req)
        except IOError, e:                  
            #We need to authenticate
            base64string = base64.encodestring('%s:%s' % (self._user, self._pass))[:-1]
            authheader =  "Basic %s" % base64string
            req.add_header("Authorization", authheader)
            try:
                handle = urllib2.urlopen(req)
            except IOError, e:                  
                #User or Pass invalid!!!
                print _("Desklicious info ::: user or pass invalid, please check your params.")
                return None
            return handle.read()   
        
        else:
            #We do not need authentication
            return handle.read()
            

class DeliciousPost:
    
    def __init__(self,data):
        self.description=data.prop('description')
        self.href=data.prop('href')
        self.tags=data.prop('tag').split(" ")
        self.extended=data.prop('extended')
